@@ -1,5 +1,5 @@
 // FbDesk.cc
-// Copyright (c) 2002 - 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -23,14 +23,16 @@
 
 #include "FbDesk.hh"
 
-#include "Font.hh"
-#include "App.hh"
-#include "EventManager.hh"
-#include "SimpleCommand.hh"
 #include "InputField.hh"
-#include "StringUtil.hh"
+#include "IntResMenuItem.hh"
 
-#include "XrmDatabaseHelper.hh"
+#include "FbTk/App.hh"
+#include "FbTk/Font.hh"
+#include "FbTk/EventManager.hh"
+#include "FbTk/SimpleCommand.hh"
+#include "FbTk/StringUtil.hh"
+#include "FbTk/XrmDatabaseHelper.hh"
+#include "FbTk/MenuSeparator.hh"
 
 #include <iostream>
 #include <fstream>
@@ -39,6 +41,7 @@
 #include <cstdio>
 
 using namespace std;
+using namespace FbTk;
 
 template <>
 void Resource<int>::setFromString(const char *str) {
@@ -105,43 +108,25 @@ FbDesk::FbDesk(const char *config_filename):
     m_button_pos_y(32),
     m_menutheme(0), // menu theme on screen 0
     m_imagectrl(0), // image control on screen 0
-    m_iconmenu(m_menutheme, 0, m_imagectrl),
-    m_inputfield(0, m_menutheme.frameFont(), m_menutheme.frameTextGC(), m_menutheme.frameTexture().color()),
+    m_iconmenu(m_menutheme, m_imagectrl),
+    m_inputfield(0, m_menutheme.frameFont(), m_menutheme.frameTextGC().gc(),
+                 m_menutheme.frameTexture().color()),
     m_textcolor("black", 0),
-    m_resmanager(),
+
+    m_resmanager(0, false),
     m_doubleclick_interval(m_resmanager, 200, "fbdesk.doubleClickInterval", "FbDesk.DoubleClickInterval"),
-    m_iconfile(m_resmanager, "~/.fluxbox/icons", "fbdesk.iconFile", "FbDesk.IconFile"),
+    m_iconfile(m_resmanager, "~/.fluxbox/fbdesk.icons", "fbdesk.iconFile", "FbDesk.IconFile"),
     m_fontname(m_resmanager, "fixed", "fbdesk.font", "FbDesk.Font"),
     m_textcolorname(m_resmanager, "black", "fbdesk.textColor", "FbDesk.TextColor"),
-    m_themefile(m_resmanager, "", "session.styleFile", "Session.StyleFile"),
     m_grid_snap_x(m_resmanager, 5, "fbdesk.snapX", "FbDesk.SnapX"),
     m_grid_snap_y(m_resmanager, 5, "fbdesk.snapY", "FbDesk.SnapY"),
-    m_textplacement(m_resmanager, Icon::TEXTPLACE_BOTTOM, "fbdesk.textPlacement", "FbDesk.TextPlacement") {
+    m_textplacement(m_resmanager, Icon::TEXTPLACE_BOTTOM, "fbdesk.textPlacement", "FbDesk.TextPlacement"),
+    m_text_alpha(m_resmanager, 0, "fbdesk.textAlpha", "FbDesk.TextAlpha"),
+    m_icon_alpha(m_resmanager, 255, "fbdesk.iconAlpha", "FbDesk.IconAlpha") {
 
-    m_resmanager.load(StringUtil::expandFilename(m_config_filename).c_str());
-
-    
-    m_textcolor.setFromString(m_textcolorname->c_str(), 0);
-    m_font.load(m_fontname->c_str());
-    
     Icon::setFont(m_font);
-    Icon::setTextColor(m_textcolor);
-    Icon::setTextPlacement(*m_textplacement);
-    if (*m_textplacement == Icon::TEXTPLACE_LEFT ||
-        *m_textplacement == Icon::TEXTPLACE_RIGHT)
-        m_font.rotate(90);
-
+    reloadConfig();
     setupIconMenu();
-#ifdef DEBUG
-    cerr<<"loadIcons"<<endl;
-#endif // DEBUG
-
-    if (!loadIcons(*m_iconfile)) {
-        cerr<<"Warning: No icon file. Creating one icon."<<endl;
-        createIcon();
-    }
-
-    loadTheme();
 
     Screen *scr = ScreenOfDisplay(FbTk::App::instance()->display(), 0);
     m_screen_width = WidthOfScreen(scr);
@@ -181,13 +166,11 @@ void FbDesk::setAntialias(bool value) {
 }
 
 void FbDesk::loadTheme() {
-    FbTk::ThemeManager::instance().load(m_themefile->c_str());
-    if (m_font.isAntialias() != m_menutheme.isAntialias())
-        m_menutheme.setAntialias(m_font.isAntialias());
-
+    FbTk::ThemeManager::instance().load(m_themefile.c_str());
+    m_menutheme.frameFont().setAntialias(m_font.isAntialias());
+    m_menutheme.titleFont().setAntialias(m_font.isAntialias());
     // update for new theme
     m_iconmenu.reconfigure();
-    m_inputfield.reconfigure(); 
 }
 
 /**
@@ -195,18 +178,16 @@ void FbDesk::loadTheme() {
 */
 bool FbDesk::loadIcons(const std::string &config_filename) {
 
-    ifstream iconfile(StringUtil::expandFilename(config_filename).c_str());
 
-    if (!iconfile)
-        return false;
+    ifstream iconfile(StringUtil::expandFilename(config_filename).c_str());
+    cerr<<"Loading: ["<<StringUtil::expandFilename(config_filename).c_str()<<"]"<<endl;
 
     if (!m_iconlist.empty()) {
         // delete old icons
         deleteIcons();
     }
 	
-    while (!iconfile.eof()) {
-		
+    while (iconfile && !iconfile.eof()) {
         string line;
         getline(iconfile, line);
         if (line != "[Desktop Entry]")
@@ -279,11 +260,10 @@ bool FbDesk::loadIcons(const std::string &config_filename) {
     } // end while
 
     // show all icons
-    IconList::iterator it = m_iconlist.begin();
-    const IconList::iterator it_end = m_iconlist.end();
-    for (; it != it_end; ++it) {
-        (*it)->show();
-    }
+    IconList::iterator icon_it = m_iconlist.begin();
+    IconList::iterator icon_it_end = m_iconlist.end();
+    for (; icon_it != icon_it_end; ++icon_it)
+        (*icon_it)->show();
 
     // create at lest one icon
     if (m_iconlist.size() == 0) {
@@ -300,7 +280,7 @@ void FbDesk::handleEvent(XEvent &event) {
         clientMessageEvent(event.xclient);
         break;
     default:
-        cerr<<"Unknown event! ("<<event.type<<")"<<endl;
+        break;
     }
 }
 
@@ -361,14 +341,13 @@ void FbDesk::buttonReleaseEvent(XButtonEvent &event) {
     m_selected_icon = 0; // unselect icon
 
     button_time = event.time; // save last timestamp
-
+    saveIcons(*m_iconfile);
 }
 
 void FbDesk::exposeEvent(XExposeEvent &event) {
     Icon *exposed_icon = findIcon(event.window);
-    if (exposed_icon != 0) {
+    if (exposed_icon != 0)
         exposed_icon->update();
-    }
 }
 
 void FbDesk::motionNotifyEvent(XMotionEvent &event) {
@@ -483,11 +462,24 @@ void FbDesk::deleteIcons() {
 }
 
 void FbDesk::reloadConfig() {
-    saveIcons(*m_iconfile); // save icons first
+
     m_resmanager.load(StringUtil::expandFilename(m_config_filename).c_str());
+    m_resmanager.loadItem(StringUtil::expandFilename(m_config_filename).c_str(),
+                          "session.styleFile", "Session.StyleFile", m_themefile);
+    if (m_themefile.empty()) {
+        m_resmanager.loadItem(StringUtil::expandFilename("~/.fluxbox/init").c_str(), 
+                              "session.styleFile", 
+                              "Session.StyleFile", m_themefile);
+    }
+    cerr<<m_themefile<<endl;
+
     m_textcolor.setFromString(m_textcolorname->c_str(), 0);
+    // setup icon values
     Icon::setTextColor(m_textcolor);
     Icon::setTextPlacement(*m_textplacement);
+    Icon::setTextAlpha(*m_text_alpha);
+    Icon::setIconAlpha(*m_icon_alpha);
+
     if (*m_textplacement == Icon::TEXTPLACE_LEFT ||
         *m_textplacement == Icon::TEXTPLACE_RIGHT)
         m_font.rotate(90);
@@ -522,6 +514,7 @@ void FbDesk::deleteLastIcon() {
 }
 
 void FbDesk::setIconCommand() {
+    m_iconmenu.hide();
     m_inputfield.setTitle("Set Command");
     if (m_last_icon != 0 &&
         find(m_iconlist.begin(), m_iconlist.end(), m_last_icon) != m_iconlist.end()) {
@@ -535,6 +528,7 @@ void FbDesk::setIconCommand() {
 }
 
 void FbDesk::setIconLabel() {
+    m_iconmenu.hide();
 
     m_inputfield.setTitle("Set Label");
     if (m_last_icon != 0 &&
@@ -552,6 +546,7 @@ void FbDesk::setLabelFromInput() {
     if (m_last_icon != 0 &&
         find(m_iconlist.begin(), m_iconlist.end(), m_last_icon) != m_iconlist.end()) {
         m_last_icon->setLabel(m_inputfield.text());
+        saveIcons(*m_iconfile);
     }
     m_inputfield.hide();
 }
@@ -560,6 +555,7 @@ void FbDesk::setCommandFromInput() {
     if (m_last_icon != 0 &&
         find(m_iconlist.begin(), m_iconlist.end(), m_last_icon) != m_iconlist.end()) { 
         m_last_icon->setCommand(m_inputfield.text());
+        saveIcons(*m_iconfile);
     }
     m_inputfield.hide();
 }
@@ -581,22 +577,38 @@ void FbDesk::setupIconMenu() {
 
     m_iconmenu.insert("Set command", set_command);
     m_iconmenu.insert("Set label", set_label);
-    m_iconmenu.insert("-------");
+    m_iconmenu.insert(new FbTk::MenuSeparator());
     m_iconmenu.insert("New icon", new_icon);
     m_iconmenu.insert("Delete icon", del_last_icon);
-    m_iconmenu.insert("-------");
+    m_iconmenu.insert(new FbTk::MenuSeparator());
+    // setup alpha menu
+    FbTk::MenuItem *item = new IntResMenuItem("Text alpha", m_text_alpha, 0, 255);
+    CommandRef update_alpha(new FbTk::SimpleCommand<FbDesk>(*this, &FbDesk::updateAlpha));
+    item->setCommand(update_alpha);
+    m_iconmenu.insert(item);
+    item = new IntResMenuItem("Icon alpha", m_icon_alpha, 0, 255);
+    item->setCommand(update_alpha);
+    m_iconmenu.insert(item);
+
     m_iconmenu.insert("Reload config", reload_style);
-    m_iconmenu.insert("-------");
+    m_iconmenu.insert(new FbTk::MenuSeparator());
     m_iconmenu.insert("Exit FbDesk", exit_fbdesk);
     //    m_iconmenu.insert("Select icon image", select_icon_image_hide);
     m_iconmenu.reconfigure(); // update graphics
 }
 
+void FbDesk::updateAlpha() {
+    m_resmanager.save(StringUtil::expandFilename(m_config_filename).c_str(),
+                      StringUtil::expandFilename(m_config_filename).c_str()); // merge
+    Icon::setTextAlpha(*m_text_alpha);
+    Icon::setIconAlpha(*m_icon_alpha);
+    updateIcons();
+}
+
 void FbDesk::updateIcons() {
-    IconList::iterator it = m_iconlist.begin();
-    IconList::iterator it_end = m_iconlist.end();
-    for (; it != it_end; ++it)
-        (*it)->update();
+    for_each(m_iconlist.begin(),
+             m_iconlist.end(),
+             mem_fun(&Icon::update));
 }
 
 }; // end namespace FbDesk
