@@ -22,12 +22,14 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ImageControl.cc,v 1.1 2003/01/09 21:09:49 fluxgen Exp $
+// $Id: ImageControl.cc,v 1.16 2004/09/10 16:41:30 akir Exp $
 
 #include "ImageControl.hh"
 
 #include "TextureRender.hh"
 #include "App.hh"
+#include "SimpleCommand.hh"
+
 
 //use GNU extensions
 #ifndef _GNU_SOURCE
@@ -43,9 +45,21 @@
 #include <sys/types.h>
 #endif // HAVE_SYS_TYPES_H
 
-#include <cstdlib>
-#include <cstring>
-#include <cstdio>
+#ifdef HAVE_CSTDLIB
+  #include <cstdlib>
+#else
+  #include <stdlib.h>
+#endif
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
+#ifdef HAVE_CSTDIO
+  #include <cstdio>
+#else
+  #include <stdio.h>
+#endif
 
 #ifdef HAVE_CTYPE_H
 #include <ctype.h>
@@ -59,10 +73,15 @@ namespace FbTk {
 
 // lookup table for texture
 unsigned long *ImageControl::sqrt_table = 0;
+#ifdef TIMEDCACHE
+bool ImageControl::s_timed_cache = true;
+#else
+bool ImageControl::s_timed_cache = false;
+#endif // TIMEDCACHE
 
 namespace { // anonymous
 
-unsigned long bsqrt(unsigned long x) {
+inline unsigned long bsqrt(unsigned long x) {
     if (x <= 0) return 0;
     if (x == 1) return 1;
 
@@ -78,11 +97,9 @@ unsigned long bsqrt(unsigned long x) {
 
 }; // end anonymous namespace
 
-
 ImageControl::ImageControl(int screen_num, bool dither,
-                             int cpc, unsigned long cache_timeout, unsigned long cmax):
+                           int cpc, unsigned long cache_timeout, unsigned long cmax):
     m_dither(dither),
-    m_timer(this),
     m_colors(0),
     m_num_colors(0),
     m_colors_per_channel(cpc) {
@@ -96,13 +113,14 @@ ImageControl::ImageControl(int screen_num, bool dither,
     m_colormap = DefaultColormap(disp, screen_num);
 
     cache_max = cmax;
-#ifdef TIMEDCACHE
-    if (cache_timeout) {
+
+    if (cache_timeout && s_timed_cache) {
         m_timer.setTimeout(cache_timeout);
+        RefCount<Command> clean_cache(new SimpleCommand<ImageControl>(*this, &ImageControl::cleanCache));
+        m_timer.setCommand(clean_cache);
         m_timer.start();
     }
-#endif // TIMEDCACHE
-	
+
     createColorTable();
 }
 
@@ -110,6 +128,7 @@ ImageControl::ImageControl(int screen_num, bool dither,
 ImageControl::~ImageControl() {
     if (sqrt_table) {
         delete [] sqrt_table;
+        sqrt_table = 0;
     }
 
     if (grad_xbuffer) {
@@ -120,45 +139,68 @@ ImageControl::~ImageControl() {
         delete [] grad_ybuffer;
     }
 
+    Display *disp = FbTk::App::instance()->display();
+
     if (m_colors) {
         unsigned long *pixels = new unsigned long [m_num_colors];
 
         for (unsigned int color = 0; color < m_num_colors; color++)
             *(pixels + color) = (*(m_colors + color)).pixel;
 
-        XFreeColors(FbTk::App::instance()->display(), m_colormap, pixels, m_num_colors, 0);
+        XFreeColors(disp, m_colormap, pixels, m_num_colors, 0);
 
         delete [] m_colors;
     }
 
     if (cache.size() > 0) {
-#ifdef DEBUG
-        cerr<<"FbTk::ImageContol: pixmap cache - releasing "<<cache.size()<<" pixmaps."<<endl;
-#endif // DEBUG
         CacheList::iterator it = cache.begin();
         CacheList::iterator it_end = cache.end();
-        Display *disp = FbTk::App::instance()->display();
         for (; it != it_end; ++it) {
             XFreePixmap(disp, (*it)->pixmap);
             delete (*it);
         }
 
     }
+
 }
 
 
 Pixmap ImageControl::searchCache(unsigned int width, unsigned int height,
-                                  unsigned long texture_type,
-                                  const FbTk::Color &color, const FbTk::Color &color_to) const {
+                                 const Texture &text) const {
+
+    if (text.pixmap().drawable() != None) {
+        // do comparsion with width/height and texture_pixmap
+        CacheList::iterator it = cache.begin();
+        CacheList::iterator it_end = cache.end();
+        for (; it != it_end; ++it) {
+            if ((*it)->texture_pixmap == text.pixmap().drawable() &&
+                (*it)->width == width && (*it)->height == height &&
+                (*it)->texture == text.type()) {
+                (*it)->count++;
+                return (*it)->pixmap;
+            }
+        }
+        return None;
+    }
+
+    /*    Cache tmp;
+    tmp.texture_pixmap = text.pixmap().drawable();
+    tmp.width = width;
+    tmp.height = height;
+    tmp.texture = text.type();
+    tmp.pixel1 = text.color().pixel();
+    tmp.pixel2 = text.colorTo().pixel();
+    */
+
     CacheList::iterator it = cache.begin();
     CacheList::iterator it_end = cache.end();
     for (; it != it_end; ++it) {
         if (((*it)->width == width) &&
             ((*it)->height == height) &&
-            ((*it)->texture == texture_type) &&
-            ((*it)->pixel1 == color.pixel())) {
-            if (texture_type & FbTk::Texture::GRADIENT) {
-                if ((*it)->pixel2 == color_to.pixel()) {
+            ((*it)->texture == text.type()) &&
+            ((*it)->pixel1 == text.color().pixel())) {
+            if (text.type() & FbTk::Texture::GRADIENT) {
+                if ((*it)->pixel2 == text.colorTo().pixel()) {
                     (*it)->count++;
                     return (*it)->pixmap;
                 }
@@ -170,23 +212,24 @@ Pixmap ImageControl::searchCache(unsigned int width, unsigned int height,
     }
 
     return None;
+
 }
 
 
 Pixmap ImageControl::renderImage(unsigned int width, unsigned int height,
-                                  const FbTk::Texture &texture) {
+                                 const FbTk::Texture &texture) {
 
     if (texture.type() & FbTk::Texture::PARENTRELATIVE)
         return ParentRelative;
 
-	// search cache first
-    Pixmap pixmap = searchCache(width, height, texture.type(),
-                                texture.color(), texture.colorTo());
-    if (pixmap)
+    // search cache first
+    Pixmap pixmap = searchCache(width, height, texture);
+    if (pixmap) {
         return pixmap; // return cache item
+    }
 
-	// render new image
-    TextureRender image(*this, width, height);
+    // render new image
+    TextureRender image(*this, width, height, m_colors, m_num_colors);
     pixmap = image.render(texture);
 
     if (pixmap) {
@@ -195,6 +238,7 @@ Pixmap ImageControl::renderImage(unsigned int width, unsigned int height,
         Cache *tmp = new Cache;
 
         tmp->pixmap = pixmap;
+        tmp->texture_pixmap = texture.pixmap().drawable();
         tmp->width = width;
         tmp->height = height;
         tmp->count = 1;
@@ -206,14 +250,10 @@ Pixmap ImageControl::renderImage(unsigned int width, unsigned int height,
         else
             tmp->pixel2 = 0l;
 
-        cache.push_back(tmp); 
+        cache.push_back(tmp);
 
-        if ((unsigned) cache.size() > cache_max) {
-#ifdef DEBUG
-            cerr<<"FbTk::ImageControl::renderImage(): cache is large, forcing cleanout"<<endl;
-#endif // DEBUG
-            timeout();
-        }
+        if ((unsigned) cache.size() > cache_max)
+            cleanCache();
 
         return pixmap;
     }
@@ -232,14 +272,15 @@ void ImageControl::removeImage(Pixmap pixmap) {
         if ((*it)->pixmap == pixmap) {
             if ((*it)->count) {
                 (*it)->count--;
-
-#ifdef TIMEDCACHE
-                timeout();
-#else // !TIMEDCACHE
-                if (! (*it)->count) timeout();
-#endif // TIMEDCACHE
+                if (s_timed_cache) {
+                    cleanCache();
+                    return;
+                }
             }
 
+            if ((*it)->count <= 0)
+                cleanCache();
+
             return;
         }
     }
@@ -247,9 +288,9 @@ void ImageControl::removeImage(Pixmap pixmap) {
 
 
 void ImageControl::colorTables(const unsigned char **rmt, const unsigned char **gmt,
-                                const unsigned char **bmt,
-                                int *roff, int *goff, int *boff,
-                                int *rbit, int *gbit, int *bbit) const {
+                               const unsigned char **bmt,
+                               int *roff, int *goff, int *boff,
+                               int *rbit, int *gbit, int *bbit) const {
 
     if (rmt) *rmt = red_color_table;
     if (gmt) *gmt = green_color_table;
@@ -272,9 +313,9 @@ void ImageControl::getXColorTable(XColor **c, int *n) {
 
 
 void ImageControl::getGradientBuffers(unsigned int w,
-                                       unsigned int h,
-                                       unsigned int **xbuf,
-                                       unsigned int **ybuf) {
+                                      unsigned int h,
+                                      unsigned int **xbuf,
+                                      unsigned int **ybuf) {
 
     if (w > grad_buffer_width) {
         if (grad_xbuffer) {
@@ -302,10 +343,10 @@ void ImageControl::getGradientBuffers(unsigned int w,
 
 
 void ImageControl::installRootColormap() {
-    XGrabServer(FbTk::App::instance()->display());
-
 
     Display *disp = FbTk::App::instance()->display();
+    XGrabServer(disp);
+
     bool install = true;
     int i = 0, ncmap = 0;
     Colormap *cmaps =
@@ -316,14 +357,14 @@ void ImageControl::installRootColormap() {
             if (*(cmaps + i) == m_colormap)
                 install = false;
         }
-		
+
         if (install)
             XInstallColormap(disp, m_colormap);
 
         XFree(cmaps);
     }
 
-    XUngrabServer(FbTk::App::instance()->display());
+    XUngrabServer(disp);
 }
 
 
@@ -339,30 +380,37 @@ unsigned long ImageControl::getSqrt(unsigned int x) const {
     if (! sqrt_table) {
         // build sqrt table for use with elliptic gradient
 
-        sqrt_table = new unsigned long[(256 * 256 * 2) + 1];
+        sqrt_table = new unsigned long[256 * 256 * 2 + 1];
         int i = 0;
 
         for (; i < (256 * 256 * 2); i++)
-            *(sqrt_table + i) = bsqrt(i);
+            sqrt_table[i] = bsqrt(i);
     }
 
-    return (*(sqrt_table + x));
+    return sqrt_table[x];
 }
 
-void ImageControl::timeout() {
+void ImageControl::cleanCache() {
     Display *disp = FbTk::App::instance()->display();
+    std::list<CacheList::iterator> deadlist;
     CacheList::iterator it = cache.begin();
     CacheList::iterator it_end = cache.end();
     for (; it != it_end; ++it) {
         Cache *tmp = (*it);
-
         if (tmp->count <= 0) {
             XFreePixmap(disp, tmp->pixmap);
-            it = cache.erase(it);
+            deadlist.push_back(it);
             delete tmp;
-            if (it == it_end) break;
+            tmp=0;
         }
     }
+
+    std::list<CacheList::iterator>::iterator dead_it = deadlist.begin();
+    std::list<CacheList::iterator>::iterator dead_it_end = deadlist.end();
+    for (; dead_it != dead_it_end; ++dead_it) {
+        cache.erase(*dead_it);
+    }
+
 }
 
 void ImageControl::createColorTable() {
@@ -417,7 +465,7 @@ void ImageControl::createColorTable() {
         }
     }
 
-    break;
+        break;
 
     case PseudoColor:
     case StaticColor: {
@@ -431,7 +479,7 @@ void ImageControl::createColorTable() {
 
         if (m_colors_per_channel < 2 || m_num_colors > static_cast<unsigned int>(1 << m_screen_depth)) {
             fprintf(stderr, "ImageControl::ImageControl: invalid colormap size %d "
-                        "(%d/%d/%d) - reducing",
+                    "(%d/%d/%d) - reducing",
                     m_num_colors, m_colors_per_channel, m_colors_per_channel,
                     m_colors_per_channel);
 
@@ -463,7 +511,7 @@ void ImageControl::createColorTable() {
                 }
             }
         }
-		
+
         for (unsigned int i = 0; i < m_num_colors; i++) {
             if (! XAllocColor(disp, m_colormap, &m_colors[i])) {
                 fprintf(stderr, "couldn't alloc color %i %i %i\n",
@@ -472,7 +520,7 @@ void ImageControl::createColorTable() {
             } else
                 m_colors[i].flags = DoRed|DoGreen|DoBlue;
         }
-		
+
         XColor icolors[256];
         unsigned int incolors = (((1 << m_screen_depth) > 256) ? 256 : (1 << m_screen_depth));
 
@@ -484,7 +532,7 @@ void ImageControl::createColorTable() {
             if (! m_colors[i].flags) {
                 unsigned long chk = 0xffffffff, pixel, close = 0;
                 char p = 2;
-		
+
                 while (p--) {
                     for (unsigned int ii = 0; ii < incolors; ii++) {
                         int r = (m_colors[i].red - icolors[i].red) >> 8;
@@ -516,97 +564,97 @@ void ImageControl::createColorTable() {
 
     case GrayScale:
     case StaticGray:
-    {
+        {
 
-        if (visual()->c_class == StaticGray) {
-            m_num_colors = 1 << m_screen_depth;
-        } else {
-            m_num_colors = m_colors_per_channel * m_colors_per_channel * m_colors_per_channel;
-
-            if (m_num_colors > static_cast<unsigned int>(1 << m_screen_depth)) {
-                m_colors_per_channel = (1 << m_screen_depth) / 3;
+            if (visual()->c_class == StaticGray) {
+                m_num_colors = 1 << m_screen_depth;
+            } else {
                 m_num_colors = m_colors_per_channel * m_colors_per_channel * m_colors_per_channel;
+
+                if (m_num_colors > static_cast<unsigned int>(1 << m_screen_depth)) {
+                    m_colors_per_channel = (1 << m_screen_depth) / 3;
+                    m_num_colors = m_colors_per_channel * m_colors_per_channel * m_colors_per_channel;
+                }
             }
-        }
 
-        if (m_colors_per_channel < 2 || m_num_colors > static_cast<unsigned int>(1 << m_screen_depth)) {
-            fprintf(stderr,"FbTk::ImageControl: invalid colormap size %d "
+            if (m_colors_per_channel < 2 || m_num_colors > static_cast<unsigned int>(1 << m_screen_depth)) {
+                fprintf(stderr,"FbTk::ImageControl: invalid colormap size %d "
                         "(%d/%d/%d) - reducing",
-                    m_num_colors, m_colors_per_channel, m_colors_per_channel,
-                    m_colors_per_channel);
+                        m_num_colors, m_colors_per_channel, m_colors_per_channel,
+                        m_colors_per_channel);
 
-            m_colors_per_channel = (1 << m_screen_depth) / 3;
-        }
+                m_colors_per_channel = (1 << m_screen_depth) / 3;
+            }
 
-        m_colors = new XColor[m_num_colors];
+            m_colors = new XColor[m_num_colors];
 
-        int p, bits = 255 / (m_colors_per_channel - 1);
-        red_bits = green_bits = blue_bits = bits;
+            int p, bits = 255 / (m_colors_per_channel - 1);
+            red_bits = green_bits = blue_bits = bits;
 
-        for (unsigned int i = 0; i < 256; i++)
-            red_color_table[i] = green_color_table[i] = blue_color_table[i] =
-                i / bits;
+            for (unsigned int i = 0; i < 256; i++)
+                red_color_table[i] = green_color_table[i] = blue_color_table[i] =
+                    i / bits;
 
-        for (unsigned int i = 0; i < m_num_colors; i++) {
-            m_colors[i].red = (i * 0xffff) / (m_colors_per_channel - 1);
-            m_colors[i].green = (i * 0xffff) / (m_colors_per_channel - 1);
-            m_colors[i].blue = (i * 0xffff) / (m_colors_per_channel - 1);;
-            m_colors[i].flags = DoRed|DoGreen|DoBlue;
-
-            if (! XAllocColor(disp, m_colormap,
-                              &m_colors[i])) {
-                fprintf(stderr, "Couldn't alloc color %i %i %i\n",
-                        m_colors[i].red, m_colors[i].green, m_colors[i].blue);
-                m_colors[i].flags = 0;
-            } else
+            for (unsigned int i = 0; i < m_num_colors; i++) {
+                m_colors[i].red = (i * 0xffff) / (m_colors_per_channel - 1);
+                m_colors[i].green = (i * 0xffff) / (m_colors_per_channel - 1);
+                m_colors[i].blue = (i * 0xffff) / (m_colors_per_channel - 1);;
                 m_colors[i].flags = DoRed|DoGreen|DoBlue;
-        }
+
+                if (! XAllocColor(disp, m_colormap,
+                                  &m_colors[i])) {
+                    fprintf(stderr, "Couldn't alloc color %i %i %i\n",
+                            m_colors[i].red, m_colors[i].green, m_colors[i].blue);
+                    m_colors[i].flags = 0;
+                } else
+                    m_colors[i].flags = DoRed|DoGreen|DoBlue;
+            }
 
 
-        XColor icolors[256];
-        unsigned int incolors = (((1 << m_screen_depth) > 256) ? 256 :
-                                 (1 << m_screen_depth));
+            XColor icolors[256];
+            unsigned int incolors = (((1 << m_screen_depth) > 256) ? 256 :
+                                     (1 << m_screen_depth));
 
-        for (unsigned int i = 0; i < incolors; i++)
-            icolors[i].pixel = i;
+            for (unsigned int i = 0; i < incolors; i++)
+                icolors[i].pixel = i;
 
-        XQueryColors(disp, m_colormap, icolors, incolors);
-        for (unsigned int i = 0; i < m_num_colors; i++) {
-            if (! m_colors[i].flags) {
-                unsigned long chk = 0xffffffff, pixel, close = 0;
+            XQueryColors(disp, m_colormap, icolors, incolors);
+            for (unsigned int i = 0; i < m_num_colors; i++) {
+                if (! m_colors[i].flags) {
+                    unsigned long chk = 0xffffffff, pixel, close = 0;
 
-                p = 2;
-                while (p--) {
-                    for (unsigned int ii = 0; ii < incolors; ii++) {
-                        int r = (m_colors[i].red - icolors[i].red) >> 8;
-                        int g = (m_colors[i].green - icolors[i].green) >> 8;
-                        int b = (m_colors[i].blue - icolors[i].blue) >> 8;
-                        pixel = (r * r) + (g * g) + (b * b);
+                    p = 2;
+                    while (p--) {
+                        for (unsigned int ii = 0; ii < incolors; ii++) {
+                            int r = (m_colors[i].red - icolors[i].red) >> 8;
+                            int g = (m_colors[i].green - icolors[i].green) >> 8;
+                            int b = (m_colors[i].blue - icolors[i].blue) >> 8;
+                            pixel = (r * r) + (g * g) + (b * b);
 
-                        if (pixel < chk) {
-                            chk = pixel;
-                            close = ii;
-                        }
+                            if (pixel < chk) {
+                                chk = pixel;
+                                close = ii;
+                            }
 
-                        m_colors[i].red = icolors[close].red;
-                        m_colors[i].green = icolors[close].green;
-                        m_colors[i].blue = icolors[close].blue;
+                            m_colors[i].red = icolors[close].red;
+                            m_colors[i].green = icolors[close].green;
+                            m_colors[i].blue = icolors[close].blue;
 
-                        if (XAllocColor(disp, m_colormap, &m_colors[i])) {
-                            m_colors[i].flags = DoRed|DoGreen|DoBlue;
-                            break;
+                            if (XAllocColor(disp, m_colormap, &m_colors[i])) {
+                                m_colors[i].flags = DoRed|DoGreen|DoBlue;
+                                break;
+                            }
                         }
                     }
                 }
             }
+
+            break;
         }
 
-        break;
-    }
-
     default:
-       cerr<<"FbTk::ImageControl: Unsupported visual"<<endl;
-       break;
+        cerr<<"FbTk::ImageControl: Unsupported visual"<<endl;
+        break;
     }
 }
 
