@@ -1,5 +1,5 @@
 // TextureRender.cc for fluxbox
-// Copyright (c) 2002 - 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
 //
 // from Image.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,16 +22,22 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: TextureRender.cc,v 1.2 2003/01/10 01:02:02 fluxgen Exp $
+// $Id: TextureRender.cc,v 1.11 2004/08/31 15:26:39 rathnor Exp $
 
 #include "TextureRender.hh"
 
 #include "ImageControl.hh"
 #include "App.hh"
+#include "FbPixmap.hh"
+#include "GContext.hh"
 
 #include <iostream>
 #include <string>
-#include <cstdio>
+#ifdef HAVE_CSTDIO
+  #include <cstdio>
+#else
+  #include <stdio.h>
+#endif
 using namespace std;
 
 namespace FbTk {
@@ -44,39 +50,40 @@ TextureRender::TextureRender(ImageControl &imgctrl,
     ncolors(ncolors),
     xtable(0), ytable(0) {
 
+
     width = ((signed) w > 0) ? w : 1;
     height = ((signed) h > 0) ? h : 1;
     // clamp to "normal" size
     if (width > 3200) {
-        cerr<<"TextureRender: Warning! Width > 3200 setting Width = 3200"<<endl;
+        cerr<<"TextureRender: "<<"Warning! Width > 3200 setting Width = 3200"<<endl;
         width = 3200;
     }
 
     if (height > 3200) {
-        cerr<<"TextureRender: Warning! Height > 3200 setting Height = 3200"<<endl;
+        cerr<<"TextureRender: "<<"Warning! Height > 3200 setting Height = 3200"<<endl;
         height = 3200;
     }
 
-    red = new (nothrow) unsigned char[width * height];
+    red = new(nothrow) unsigned char[width * height];
     if (red == 0) {
         char sbuf[128];
         sprintf(sbuf, "%d", width*height);
-        throw string("TextureRender::TextureRender(): Out of memory while allocating red buffer."+ string(sbuf));
+        throw string("TextureRender::TextureRender(): " + string("Out of memory while allocating red buffer.") + string(sbuf));
     }
 
 
-    green = new (nothrow)  unsigned char[width * height];
+    green = new(nothrow) unsigned char[width * height];
     if (green == 0) {
         char sbuf[128];
         sprintf(sbuf, "%d", width*height);
-        throw string("TextureRender::TextureRender(): Out of memory while allocating green buffer. size " + string(sbuf));
+        throw string("TextureRender::TextureRender(): " + string("Out of memory while allocating red buffer.") + string(sbuf));
     }
 
-    blue = new (nothrow) unsigned char[width * height];
+    blue = new(nothrow) unsigned char[width * height];
     if (blue == 0) {
         char sbuf[128];
         sprintf(sbuf, "%d", width*height);
-        throw string("TextureRender::TextureRender(): Out of memory while allocating blue buffer. size " + string(sbuf));
+        throw string("TextureRender::TextureRender(): " + string("Out of memory while allocating red buffer.") + string(sbuf));
     }
 
     cpc = imgctrl.colorsPerChannel();
@@ -97,8 +104,9 @@ TextureRender::~TextureRender() {
 
 
 Pixmap TextureRender::render(const FbTk::Texture &texture) {
-
-    if (texture.type() & FbTk::Texture::PARENTRELATIVE)
+    if (texture.pixmap().drawable() != 0)
+        return renderPixmap(texture);
+    else if (texture.type() & FbTk::Texture::PARENTRELATIVE)
         return ParentRelative;
     else if (texture.type() & FbTk::Texture::SOLID)
         return renderSolid(texture);
@@ -111,96 +119,86 @@ Pixmap TextureRender::render(const FbTk::Texture &texture) {
 
 Pixmap TextureRender::renderSolid(const FbTk::Texture &texture) {
 
-    Display *disp = FbTk::App::instance()->display();
+    FbPixmap pixmap(RootWindow(FbTk::App::instance()->display(),
+                               control.screenNumber()), 
+                    width, height,
+                    control.depth());
 
-    Pixmap pixmap = XCreatePixmap(disp,
-                                  RootWindow(disp, control.screenNum()), width,
-                                  height, control.depth());
-    if (pixmap == None) {
-        cerr<<"FbTk::TextureRender::render_solid(): error creating pixmap"<<endl;
+    if (pixmap.drawable() == None) {
+        cerr<<"FbTk::TextureRender::render_solid(): Error creating pixmap"<<endl;
         return None;
     }
 
-    XGCValues gcv;
-    GC gc, hgc, lgc;
 
-    gcv.foreground = texture.color().pixel();
-    gcv.fill_style = FillSolid;
-    gc = XCreateGC(disp, pixmap, GCForeground | GCFillStyle, &gcv);
+    FbTk::GContext gc(pixmap), 
+        hgc(pixmap), lgc(pixmap);
 
-    gcv.foreground = texture.hiColor().pixel();
-    hgc = XCreateGC(disp, pixmap, GCForeground, &gcv);
+    gc.setForeground(texture.color());
+    gc.setFillStyle(FillSolid);
 
-    gcv.foreground = texture.loColor().pixel();
-    lgc = XCreateGC(disp, pixmap, GCForeground, &gcv);
+    hgc.setForeground(texture.hiColor());
 
-    XFillRectangle(disp, pixmap, gc, 0, 0, width, height);
+    pixmap.fillRectangle(gc.gc(), 0, 0, width, height);
 	
     using namespace FbTk;
 	
     if (texture.type() & Texture::INTERLACED) {
-        gcv.foreground = texture.colorTo().pixel();
-        GC igc = XCreateGC(disp, pixmap,
-                           GCForeground, &gcv);
-
+        lgc.setForeground(texture.colorTo());
         register unsigned int i = 0;
         for (; i < height; i += 2)
-            XDrawLine(disp, pixmap, igc, 0, i, width, i);
+            pixmap.drawLine(lgc.gc(), 0, i, width, i);
 
-        XFreeGC(disp, igc);
     }
 
+    lgc.setForeground(texture.loColor());
+
     if (texture.type() & Texture::BEVEL1) {
         if (texture.type() & Texture::RAISED) {
-            XDrawLine(disp, pixmap, lgc,
-                      0, height - 1, width - 1, height - 1);
-            XDrawLine(disp, pixmap, lgc,
-                      width - 1, height - 1, width - 1, 0);
+            pixmap.drawLine(lgc.gc(), 
+                            0, height - 1, width - 1, height - 1);
+            pixmap.drawLine(lgc.gc(),
+                            width - 1, height - 1, width - 1, 0);
 
-            XDrawLine(disp, pixmap, hgc,
-                      0, 0, width - 1, 0);
-            XDrawLine(disp, pixmap, hgc,
-                      0, height - 1, 0, 0);
+            pixmap.drawLine(hgc.gc(),
+                            0, 0, width - 1, 0);
+            pixmap.drawLine(hgc.gc(),
+                            0, height - 1, 0, 0);
         } else if (texture.type() & Texture::SUNKEN) {
-            XDrawLine(disp, pixmap, hgc,
-                      0, height - 1, width - 1, height - 1);
-            XDrawLine(disp, pixmap, hgc,
-                      width - 1, height - 1, width - 1, 0);
+            pixmap.drawLine(hgc.gc(),
+                            0, height - 1, width - 1, height - 1);
+            pixmap.drawLine(hgc.gc(),
+                            width - 1, height - 1, width - 1, 0);
 
-            XDrawLine(disp, pixmap, lgc,
-                      0, 0, width - 1, 0);
-            XDrawLine(disp, pixmap, lgc,
-                      0, height - 1, 0, 0);
+            pixmap.drawLine(lgc.gc(),
+                            0, 0, width - 1, 0);
+            pixmap.drawLine(lgc.gc(),
+                            0, height - 1, 0, 0);
         }
     } else if (texture.type() & Texture::BEVEL2) {
         if (texture.type() & Texture::RAISED) {
-            XDrawLine(disp, pixmap, lgc,
-                      1, height - 3, width - 3, height - 3);
-            XDrawLine(disp, pixmap, lgc,
-                      width - 3, height - 3, width - 3, 1);
+            pixmap.drawLine(lgc.gc(),
+                            1, height - 3, width - 3, height - 3);
+            pixmap.drawLine(lgc.gc(),
+                            width - 3, height - 3, width - 3, 1);
 
-            XDrawLine(disp, pixmap, hgc,
-                      1, 1, width - 3, 1);
-            XDrawLine(disp, pixmap, hgc,
-                      1, height - 3, 1, 1);
+            pixmap.drawLine(hgc.gc(),
+                            1, 1, width - 3, 1);
+            pixmap.drawLine(hgc.gc(),
+                            1, height - 3, 1, 1);
         } else if (texture.type() & Texture::SUNKEN) {
-            XDrawLine(disp, pixmap, hgc,
-                      1, height - 3, width - 3, height - 3);
-            XDrawLine(disp, pixmap, hgc,
-                      width - 3, height - 3, width - 3, 1);
+            pixmap.drawLine(hgc.gc(),
+                            1, height - 3, width - 3, height - 3);
+            pixmap.drawLine(hgc.gc(),
+                            width - 3, height - 3, width - 3, 1);
 
-            XDrawLine(disp, pixmap, lgc,
-                      1, 1, width - 3, 1);
-            XDrawLine(disp, pixmap, lgc,
-                      1, height - 3, 1, 1);
+            pixmap.drawLine(lgc.gc(),
+                            1, 1, width - 3, 1);
+            pixmap.drawLine(lgc.gc(),
+                            1, height - 3, 1, 1);
         }
     }
 
-    XFreeGC(disp, gc);
-    XFreeGC(disp, hgc);
-    XFreeGC(disp, lgc);
-
-    return pixmap;
+    return pixmap.release();
 }
 
 
@@ -257,16 +255,35 @@ Pixmap TextureRender::renderGradient(const FbTk::Texture &texture) {
 
 }
 
+Pixmap TextureRender::renderPixmap(const FbTk::Texture &src_texture) {
+    if (width != src_texture.pixmap().width() ||
+        height != src_texture.pixmap().height()) {
+
+        // copy src_texture's pixmap and 
+        // scale/tile to fit our size
+        FbPixmap new_pm(src_texture.pixmap());
+ 
+        if ((src_texture.type() & Texture::TILED)) {
+            new_pm.tile(width,height);
+        } else {
+            new_pm.scale(width, height);
+        }
+
+        return new_pm.release();
+    }
+    // return copy of pixmap
+    return FbPixmap(src_texture.pixmap()).release();
+}
 
 XImage *TextureRender::renderXImage() {
     Display *disp = FbTk::App::instance()->display();
     XImage *image =
         XCreateImage(disp,
-                     DefaultVisual(disp, control.screenNum()), control.depth(), ZPixmap, 0, 0,
+                     DefaultVisual(disp, control.screenNumber()), control.depth(), ZPixmap, 0, 0,
                      width, height, 32, 0);
 
     if (! image) {
-        cerr<<"FbTk::TextureRender::renderXImage(): error creating XImage"<<endl;
+        cerr<<"FbTk::TextureRender::renderXImage(): Error creating XImage"<<endl;
         return 0;
     }
 
@@ -488,12 +505,12 @@ XImage *TextureRender::renderXImage() {
             nberr = terr;
         }
 
-        delete [] rerr;
-        delete [] gerr;
-        delete [] berr;
-        delete [] nrerr;
-        delete [] ngerr;
-        delete [] nberr;
+            delete [] rerr;
+            delete [] gerr;
+            delete [] berr;
+            delete [] nrerr;
+            delete [] ngerr;
+            delete [] nberr;
 #endif // ORDEREDPSUEDO
 
         } break;
@@ -650,28 +667,25 @@ return image;
 
 Pixmap TextureRender::renderPixmap() {
     Display *disp = FbTk::App::instance()->display();
-    Pixmap pixmap =
-        XCreatePixmap(disp,
-                      RootWindow(disp, control.screenNum()), width, height, control.depth());
+    FbPixmap pixmap(RootWindow(disp, control.screenNumber()), 
+                    width, height, control.depth());
 
-    if (pixmap == None) {
-        cerr<<"TextureRender::renderPixmap(): error creating pixmap"<<endl;
+    if (pixmap.drawable() == None) {
+        cerr<<"FbTk::TextureRender::renderPixmap(): Error creating pixmap"<<endl;
         return None;
     }
 
     XImage *image = renderXImage();
 
     if (! image) {
-        XFreePixmap(disp, pixmap);
         return None;
     } else if (! image->data) {
         XDestroyImage(image);
-        XFreePixmap(disp, pixmap);
         return None;
     }
 
-    XPutImage(disp, pixmap,
-              DefaultGC(disp, control.screenNum()),
+    XPutImage(disp, pixmap.drawable(),
+              DefaultGC(disp, control.screenNumber()),
               image, 0, 0, 0, 0, width, height);
 
     if (image->data != 0) {
@@ -681,7 +695,7 @@ Pixmap TextureRender::renderPixmap() {
 
     XDestroyImage(image);
 
-    return pixmap;
+    return pixmap.release();
 }
 
 
